; ##############################################################################
; ##############################################################################
; ##############################################################################
EXTERNER_INTERUPT_INITIALISIEREN:
;																			
; SLEEP+INT0+INT1 (externer Interrupt) konfigurieren						
;																			
; MCUCR => SE SM2 SM1 SM0 ISC11 ISC10 ISC01 ISC00 							
; SM2 SM1 SM0 Sleep Mode													
;*0   0   0   Idle															
; 0   0   1   ADC Noise Reduction											
; 0   1   0   Power-down													
; 0   1   1   Power-save													
; 1   0   0   Reserved														
; 1   0   1   Reserved														
; 1   1   0   Standby(1)													
;																			
;ISC11 ISC10 Description													
; 0 	0 	The low level of INT1 generates an interrupt request.			
; 0 	1 	Any logical change on INT1 generates an interrupt request.		
; 1 	0 	The falling edge of INT1 generates an interrupt request.		
; 1 	1 	The rising edge of INT1 generates an interrupt request.			
;																			
;ISC01 ISC00 Description													
; 0 	0 	The low level of INT0 generates an interrupt request.			
;*0 	1 	Any logical change on INT0 generates an interrupt request.		
; 1 	0 	The falling edge of INT0 generates an interrupt request.		
; 1 	1 	The rising  edge of INT0 generates an interrupt request.		
;																			
	ldi temp,(1<<SE)|(0<<SM2)|(0<<SM1)|(0<<SM0)|(0<<ISC11)|(0<<ISC10)|(0<<ISC01)|(1<<ISC00) 
	out MCUCR,temp

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
EXTERNER_INTERRUPT_AKTIVIEREN:
;																			
; INT0+INT1 (externer Interrupt)aktivieren									
;																			
; GICR => INT1 INT0 - - - - IVSEL IVCE 										
; General Interrupt Control													
;																			
	ldi temp, (0<<INT1)|(1<<INT0)|(0<<IVSEL)|(0<<IVCE)   
    out GICR, temp 
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Dieser Interrupt wird bei jedem Flankenwechsel des 57kHz Signals aufgerufen.	
; Der RDS-DATA-PIN und RDS-TAKT-PIN geschaltet.									
; Der RDS
; 57.000Hz:48=1.187,5 Hz														
RDS_GENERATOR:
	out TCNT0,TIMER0WERT; berwachung / TIMER0 Startwert		
	push TEMP		    ; Sichern von "TEMP" im Stack
	in	 TEMP,SREG		; Einlesen des SREG 
	push TEMP	    	; Schreiben von  SREG  im Stack (KOPIE)
	push ZL
	push ZH
;-------------------------------------------------------------------------------
; ANALOG-AUSGANG (Signal invertieren / nicht invertieren)						
	tst INVERTIERUNG
	breq RDS_GENERATOR_NO_INVERTIERUNG
	sbis (AUSGANG_57kHz_PIN),(AUSGANG_57kHz_PIN_NR)	; springe, wenn BIT(PIN) high	
	rjmp RDS_GENERATOR_PIN_HIGH
	rjmp RDS_GENERATOR_PIN_LOW
RDS_GENERATOR_NO_INVERTIERUNG:
	sbis (AUSGANG_57kHz_PIN),(AUSGANG_57kHz_PIN_NR)	; springe, wenn BIT(PIN) high	
	rjmp RDS_GENERATOR_PIN_LOW
RDS_GENERATOR_PIN_HIGH:
	sbi (AUSGANG_RDS_ANALOG_PORT),(AUSGANG_RDS_ANALOG_PIN_NR); Analog 57kHz	
	rjmp RDS_GENERATOR_PIN_FERTIG
RDS_GENERATOR_PIN_LOW:
	cbi (AUSGANG_RDS_ANALOG_PORT),(AUSGANG_RDS_ANALOG_PIN_NR); Analog 57kHz	
RDS_GENERATOR_PIN_FERTIG:
;-------------------------------------------------------------------------------
; RDS DATA (Daten aus dem SRAM laden)											
	tst RDS_ZAEHLER
	brne RDS_GENERATOR_DATA_FERTIG
	clr DATA
	; Z-Initalisieren	
	ldi ZL,LOW (adr_RDS_DATA_BUFFER13)
	ldi ZH,HIGH(adr_RDS_DATA_BUFFER13)
	; Aufbereitung Z	
	mov temp,RDS_ZAEHLER_BITS
	LSR temp	; 3x rechts			
	LSR temp
	LSR temp
	add ZL,temp	; addieren			
	adc ZH,NULL
	LD temp,Z	; aus SRAM laden	
	LSL temp	; 1 x links (Carry rollt raus)	
	ST Z,temp	; im SRAM ablegen	
	ROR DATA 	; 1 x rechts (Carry rollt rein)	
	tst DATA
	brne RDS_GENERATOR_DATA_HIGH
RDS_GENERATOR_DATA_LOW:
	cbi (AUSGANG_RDS_DATA_PORT)  ,(AUSGANG_RDS_DATA_PIN_NR)	; RDS-DATA		
	rjmp RDS_GENERATOR_DATA_FERTIG
RDS_GENERATOR_DATA_HIGH:
	sbi (AUSGANG_RDS_DATA_PORT)  ,(AUSGANG_RDS_DATA_PIN_NR)	; RDS-DATA		
RDS_GENERATOR_DATA_FERTIG:
;-------------------------------------------------------------------------------
; AUSGANG-RDS_TAKT																
	mov temp,RDS_ZAEHLER
	cpi temp,(1)
	brne RDS_GENERATOR_TAKT1_FERTIG
	sbi (AUSGANG_RDS_TAKT_PORT)  ,(AUSGANG_RDS_TAKT_PIN_NR)	; RDS-TAKT (HIGH)	
RDS_GENERATOR_TAKT1_FERTIG:
;-------------------------------------------------------------------------------
; AUSGANG-RDS_TAKT																
	mov temp,RDS_ZAEHLER
	cpi temp,(48)
	brne RDS_GENERATOR_TAKT2_FERTIG
	cbi (AUSGANG_RDS_TAKT_PORT)  ,(AUSGANG_RDS_TAKT_PIN_NR)	; RDS-TAKT (LOW)	
RDS_GENERATOR_TAKT2_FERTIG:
;-------------------------------------------------------------------------------
; ANALOG-AUSGANG																
; Wenn DATA=NULL dann wird SIGNAL Invertiert									
	mov temp,RDS_ZAEHLER
	cpi temp,(47)
	brne RDS_GENERATOR_ANALOG_DATA_FERTIG
	tst DATA
	brne RDS_GENERATOR_ANALOG_DATA_FERTIG
	com INVERTIERUNG
RDS_GENERATOR_ANALOG_DATA_FERTIG:
;-------------------------------------------------------------------------------
	; RDS_ZAEHLER 0...95		
	inc RDS_ZAEHLER
	mov temp,RDS_ZAEHLER
	cpi temp,(95+1)
	brlo RDS_GENERATOR_ZAEHLER_OK	
	clr RDS_ZAEHLER
	; INVERTIERUNG				
	com INVERTIERUNG
	; RDS_ZAEHLER_BITS 0...103	
	inc RDS_ZAEHLER_BITS
	mov temp,RDS_ZAEHLER_BITS
	cpi temp,(103+1)
	brlo RDS_GENERATOR_ZAEHLER_OK
	clr RDS_ZAEHLER_BITS
RDS_GENERATOR_ZAEHLER_OK:
;-------------------------------------------------------------------------------
	pop ZH
	pop ZL
	pop	TEMP			; LESEN von SREG vom STACK (KOPIE)
	out	SREG,TEMP		; Wiederherstellen von SREG
	pop	TEMP			; Wiederherstellen von "TEMP" 			
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
